//
// Simple OpenAL wrapper
//
// Author: Alex V. Boreskoff <steps3d@narod.ru>, <alexboreskoff@mtu-net.ru>
//

#ifndef	__OPENAL_CONTEXT__
#define	__OPENAL_CONTEXT__

#ifdef	MACOSX
	#include	<OpenAL/al.h>
	#include	<OpenAL/alc.h>
#else
	#include	<al.h>
	#include	<alc.h>
#endif

#include	<stdio.h>
#include	"Vector3D.h"

class	SoundData;
class	SoundSource;

class	OpenAlContext
{
protected:
	ALCdevice  * device;					// device & context for working with OpenAL
	ALCcontext * context;
	Vector3D	 pos;						// listener's position
	Vector3D	 velocity;					// listeners velocity (for Doppler effect)
	Vector3D	 viewDir;					// listener's orientation - view and up vectors
	Vector3D	 upDir;

public:
	OpenAlContext  ();
	~OpenAlContext ();

	bool	isOk () const
	{
		return device != NULL && context != NULL;
	}

	const Vector3D&	getPos () const
	{
		return pos;
	}

	const Vector3D&	getVelocity () const
	{
		return velocity;
	}

	const Vector3D& getViewDir () const
	{
		return viewDir;
	}

	const Vector3D&	getUpDir () const
	{
		return upDir;
	}

	void	moveTo         ( const Vector3D& newPos );
	void	moveBy         ( const Vector3D& delta  );
	void	setVelocity    ( const Vector3D& newVel );
	void	setOrientation ( const Vector3D& view, const Vector3D& up );
	
	static	SoundData * open            ( const char * fileName );
	static	bool		registerSound   ( SoundSource * sound );
	static	bool		unregisterSound ( SoundSource * sound );
	static	void		updateSounds    ();
};

#endif

