//
// Sound source wrapper for OpenAL
//
// Author: Alex V. Boreskoff
//

#ifndef	__SOUND_SOURCE__
#define	__SOUND_SOURCE__

#ifdef	MACOSX
	#include	<OpenAL/al.h>
	#include	<OpenAL/alc.h>
#else
	#include	<al.h>
	#include	<alc.h>
	#include	<alut.h>
#endif

#include	"Vector3D.h"
#include	"TypeDefs.h"					// for byte, uint16 & etc

class	Data;
class	SoundData;

class	SoundSource
{
protected:
	SoundData * dataSource;
	bool		looping;
	bool		stream;
	ALuint		format;
	ALuint		source;
	ALuint		buffer [2];						// id's of OpenAL buffers
	int			currentBuffer;
	byte      * buf;							// buffer for sound data
	Vector3D	pos;						 	// sound source position
	Vector3D	velocity;						// sound source velocity
	float		pitch;
	float		gain;
	bool		   mono;						// whether mono mode forced

public:
	SoundSource  ( SoundData * data, bool theLoop = false, bool forceMono = false );
	~SoundSource ();

	const Vector3D&	getPos () const
	{
		return pos;
	}

	const Vector3D&	getVelocity () const
	{
		return velocity;
	}

	float	getGain () const
	{
		return gain;
	}

	float	getPitch () const
	{
		return pitch;
	}

	bool	getLooping () const
	{
		return looping;
	}

	bool	isPlaying () const;
	bool	atEnd     () const;
	
	void	setPos      ( const Vector3D& newPos );
	void	setVelocity ( const Vector3D& newVel );
	void	setPitch    ( float newPitch );
	void	setGain     ( float newGain  );
	void	setLooping  ( bool flag      );

	void	play   ();
	void	pause  ();
	void	stop   ();
	void	rewind ();
	void	update ();

	enum
	{
		BUFFER_SIZE = 65536
	};

											// return sound format (OpenAL) for this data
	ALuint	soundFormat ( int numChannels, int bitsPerSample ) const;
};

#endif
